/***************************************************************
 * stack.h ---- header file for proj1.c and stack.c            *
 *                                                             *
 ***************************************************************/

#ifndef STACK_H

#define STACK_H

/* link list structure */
struct link_list {    
  char value;          
  struct link_list *next;
};

/* stack operation entries (Stack class structure) */
struct stack_common {  
  void (* push_op)(struct link_list **top,char element);    /* pointer to function push */
  char (* pop_op)(struct link_list **top);                  /* pointer to function pop  */
  void (* show_op)(struct link_list *top);                  /* pointer to function show */
  struct link_list (* scan_op)(struct link_list *top, char element);  /* pointer to function scan */
  void (* copy_op)(struct link_list *top_from, struct link_list **top_to); 
};

/* stack structure (instance structure) --- interface to define instance of the Stack class*/
typedef struct _Stack {   /*  Stack of integers */
  struct stack_common *stack_ops;  /* pointer to data structure of stack class (common part of the class) */
  struct link_list *stack_top;     /* instance-specific data (stack top pointor) */
} Stack;

/* function prototypes */
void push(struct link_list **top,char element);
char pop(struct link_list **top);
void show(struct link_list *top);          
struct link_list * scan(struct link_list *top, char element);
void copy(struct link_list *top_from, struct link_list **top_to); 

/* extern variables */
extern struct stack_common CLASS; 

/* macro definitions --- interface to do operations on the class instances(objects) */
#define Stack_Push(x,y) (*(((x).stack_ops)->push_op))(&((x).stack_top),y) /* Push y into stack x */
#define Stack_Pop(x) (*(((x).stack_ops)->pop_op))(&((x).stack_top))       /* Pop from stack x(return NULL if x is empty) */
#define Stack_Show(x) (*(((x).stack_ops)->show_op))((x).stack_top)        /* Show all elements in stack x from top to bottom */
#define Stack_Scan(x) (*(((x).stack_ops)->show_op))((x).stack_top,(y))    /* Query the first location from top of an element y 
									     on stack x(return NULL if y is not in x)  */
#define Stack_Copy(from,to) (*(((from).stack_ops)->copy_op))((from).stack_top,&((to).stack_top))  
#define Stack_Create(x) { (x).stack_ops = &CLASS; (x).stack_top = NULL;}  /* Create stack x  */
#define Stack_Destroy(x) { while (Stack_Pop(x)); (x).stack_ops = NULL; }  /* destroy stack x (free space if any) */

#endif STACK_H



